# Exemple d'utilisation de matplotlib

# On utilise numpy.random.rand() pour generer des points mais en realite
# il faudra programmer son propre generateur aleatoire sans utiliser numpy.random

import sys
Windows = 'win' in sys.platform
import matplotlib.pyplot as plt
if not(Windows):
    from matplotlib.pyplot import ion
    ion()

def histogramme(l, titre='histogramme'):
    plt.figure()
    plt.title(titre)
    plt.hist(l)
    plt.show()
    return(plt)

